"""
AR Amplifier module

Copyright 2018 Amplifier Research
"""

import time
import visa
import re
from .ampmodels import ampmodeldict
from .errors import QueryError

# Constants

# Amplifier Mode
MODE_MANUAL     = 0
MODE_PULSE      = 1
MODE_ALC_INT    = 2
MODE_ALC_EXT    = 3

# Amplifier Band
BAND_LOW        = 0
BAND_HIGH       = 1
BAND_AUTO       = 2

# VSWR Limit
VSWR_LIMIT_NONE = 0
VSWR_LIMIT_6TO1 = 1
VSWR_LIMIT_3TO1 = 2
VSWR_LIMIT_2TO1 = 3

# Command sets (for use in AR_Amp class)
CS_LONG_RF_MAN  = 0
CS_LONG_RF_CW   = 1
CS_LONG_OP_MAN  = 2
CS_LONG_OP_CW   = 3
CS_4LINE        = 4
CS_SHORT        = 5
CS_2LINE        = 6

class AR_Amp:
    """Class for handling VISA communication with AR amplifiers."""

    def __init__(self, model=None):
        """
        Initializes a new AR_Amp instance.

        If no model is given or if the supplied model is not found,
        defaults to the long command set for newer amps.
        """
        try:
            self.model = model
            self._commandset = ampmodeldict[model]
        except KeyError:
            self.model = None
            self._commandset = CS_LONG_RF_MAN

    def all_models(self):
        """Returns a list of all supported models."""
        return list(ampmodeldict.keys())

    # VISA communication methods

    def initialize(self, VISAResource,
        baudRate = 9600,
        flowControl = visa.constants.VI_ASRL_FLOW_NONE,
        parity = visa.constants.VI_ASRL_PAR_NONE,
        dataBits = 8,
        stopBits = visa.constants.VI_ASRL_STOP_ONE):
        """
        Opens a VISA session to the amplifier.
        
        If VISAResource is a serial port, it will be configured using the supplied parameters.
        flowControl, parity, and stopBits should use constants from visa.constants.
        """
        self._instr = visa.ResourceManager().open_resource(VISAResource, query_delay=0.05, write_termination='\n')

        self._instr.read_termination = '\n'

        # Configure serial if necessary
        if self._instr.interface_type == visa.constants.InterfaceType.asrl:
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_BAUD, baudRate)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_FLOW_CNTRL, flowControl)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_PARITY, parity)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_DATA_BITS, dataBits)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_STOP_BITS, stopBits)

    def reset(self):
        """Resets the amplifier, clears faults."""
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            command = 'R'
        else:
            command = 'RESET'

        self._instr.write(command)

    def id_query(self):
        """Returns the identity string from the amp."""
        if self._commandset == CS_2LINE:
            return
        elif self._commandset == CS_4LINE:
            return self._instr.query('I?')
        else:
            return self._instr.query('*IDN?')

    def close(self):
        """Closes the VISA session."""
        self._instr.close()

    def read_state(self):
        """Reads the state of the amplifier.

        Not supported on some legacy amplifiers. Refer to your manual
        to determine if this command is supported and view the table that
        contains descriptions of each bit in the response.
        """
        if self._commandset in (CS_4LINE, CS_2LINE):
            return 0
        else:
            response = self._instr.query('STATE?')
            if response[0:6] == 'STATE=':
                return int(response.split('=')[1], 16)
            else:
                raise QueryError(response)

    def read_fault(self):
        """Queries the amplifier for faults.
        
        Refer to your manual to determine the fault codes and descriptions for your amplifier.
        """
        if self._commandset in (CS_4LINE, CS_2LINE):
            return self._instr.read_stb()
        else:
            response = self._instr.query('FSTA?')
            if response[0:5] == 'FSTA=':
                return int(response.split('=')[1], 16)
            else:
                raise QueryError(response)

    def read_fault_log_count(self):
        """Queries the total number of faults currently stored in the fault log.
        
        *Only valid on amplifiers that support fault logging.
        """
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return 0
        else:
            return int(self._instr.query('FSTA:LOG:COUNT?'))

    def read_fault_log(self, fault_number):
        """Queries a single entry within the fault log. The fault log stores up to 200 of the most recent faults. 
        Each fault is tagged with a decimal number between 1 (most recent) and 200 (oldest). 
        If the fault log exceeds 200 faults, the oldest fault in the list will be cleared.

        Returns a tuple of integers in the form (fault code, power on Hours, RF on hours)

        *Only valid on amplifiers that support fault logging.
        """
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return 0, 0, 0
        else:
            response = self._instr.query('FSTA:LOG {}?').format(fault_number)
            splitresponse = response.split(',')
            if len(splitresponse) == 3:                
                return tuple((int(splitresponse[0], 16), int(splitresponse[1]), int(splitresponse[2])))
            else:
                return 0, 0, 0

    def clear_fault_log(self):
        """Clears the entire contents of the error log. This action is permanent and cannot be undone.
        
        *Only valid on amplifiers that support fault logging.
        """
        if self._commandset not in (CS_4LINE, CS_SHORT, CS_2LINE):
            self._instr.write('FSTA:LOG:CLEAR')
            time.sleep(0.250)
            
    def power(self, state):
        """Turns the amplifier power on or off.
        
        The state parameter will accept 1, 'on', or True to turn the power on.
        """
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            if state in (1, 'on', True):
                command = 'P1'
            else:
                command = 'P0'
        else:
            if state in (1, 'on', True):
                command = 'POWER:ON'
            else:
                command = 'POWER:OFF'
        
        self._instr.write(command)
        if state in (1, 'on', True):
            time.sleep(2)
    
    def rf_output(self, state):
        """Turns the RF output on or off.
        
        The state parameter will accept 1, 'on', or True to turn the RF on.
        """
        if self._commandset in (CS_SHORT, CS_2LINE):
            return
        elif self._commandset in (CS_LONG_OP_MAN, CS_LONG_OP_CW):
            if state in (1, 'on', True):
                command = 'OPERATE'
            else:
                command = 'STANDBY'
        elif self._commandset == CS_4LINE:
            if state in (1, 'on', True):
                command = 'O'
            else:
                command = 'S'
        else:
            if state in (1, 'on', True):
                command = 'RF:ON'
            else:
                command = 'RF:OFF'

        self._instr.write(command)

    def set_rf_gain(self, rf_gain):
        """Sets the RF gain of the amplifier between 0 and 100."""
        # Clamp the gain between 0 and 100
        rf_gain = max(0, min(int(rf_gain), 100))

        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            self._instr.write('G{0:04d}'.format(int(rf_gain / 100 * 4095)))
        else:
            self._instr.write('LEVEL:GAIN{}'.format(rf_gain))

    def set_mode(self, mode):
        """Sets the mode of the amplifier to manual, pulse, or ALC.

        Use the MODE_ constants in this module for readability.
        Refer to your manual to determine which modes are supported."""
        # Do nothing if an incorrect mode was passed in
        if mode not in (MODE_MANUAL, MODE_PULSE, MODE_ALC_INT, MODE_ALC_EXT):
            return

        if self._commandset in (CS_SHORT, CS_2LINE):
            return

        elif self._commandset == CS_4LINE:
            command = ('MM', 'MP', 'MA', 'MA')[mode]

            self._instr.write(command)

            if mode == MODE_ALC_INT:
                self._instr.write('DI')
            elif mode == MODE_ALC_EXT:
                self._instr.write('DE')

        else: # Long commands
            if self._commandset in (CS_LONG_RF_MAN, CS_LONG_OP_MAN):
                command = ('MODE:MANUAL', 'MODE:PULSE', 'MODE:ALC INT', 'MODE:ALC EXT')[mode]
            else:
                command = ('MODE:CW', 'MODE:PULSE', 'MODE:ALC INT', 'MODE:ALC EXT')[mode]

            self._instr.write(command)

    def set_band(self, band):
        """Switches between low and high frequency band on dual band amplifiers.

        Use the BAND_ constants in this module for readability.
        *Note: Auto band selection is only supported on a small number of legacy amplifiers.
        """ 
        # Do nothing if incorrect band passed in
        if band not in (BAND_LOW, BAND_HIGH, BAND_AUTO):
            return

        if self._commandset == CS_SHORT:
            if band == BAND_HIGH:
                command = 'BANDH'
            else:
                command = 'BANDL'
        elif self._commandset == CS_2LINE:
            command = ('BL', 'BH', 'BA')[band]
        else:
            return

        self._instr.write(command)

    def set_alc_detector_gain(self, det_gain):
        """Sets the ALC detector gain between 0 and 100."""
        # Clamp the gain between 0 and 100
        det_gain = max(0, min(int(det_gain), 100))

        if self._commandset in (CS_SHORT, CS_2LINE):
            return
        elif self._commandset == CS_4LINE:
            self._instr.write('D{0:03d}'.format(int(det_gain / 100 * 255)))
        else:
            self._instr.write('LEVEL:DET{}'.format(det_gain))

    def set_alc_response(self, response):
        """Sets the ALC response time.
        
        A lower value corresponds to a faster ALC loop frequency.
        This method will coerce the input into the valid range for this value, which is as follows:
        Newer LCD touch panel amplifiers: 1-7
        Legacy amplifiers: 0-6
        """
        if self._commandset in (CS_SHORT, CS_2LINE):
            return
        elif self._commandset == CS_4LINE:
            command = 'LR{}'.format(max(0, min(6, int(response))))
        else:
            command = 'LEVEL:RESP{}'.format(max(1, min(7, int(response))))

        self._instr.write(command)

    def set_alc_threshold(self, threshold):
        """Sets the ALC threshold level from 0 to 100."""
        # Clamp the threshold between 0 and 100
        threshold = max(0, min(100, int(threshold)))

        if self._commandset in (CS_SHORT, CS_2LINE):
            return
        elif self._commandset == CS_4LINE:
            self._instr.write('T{0:04d}'.format(int(threshold / 100 * 4095)))
        else:
            self._instr.write('LEVEL:THR{}'.format(threshold))

    def set_vswr_limit(self, limit):
        """Set the value of the output VSWR limit.  If the limit is exceeded, a fault will be triggered, and the RF output will be disabled."""
        # Do nothing if incorrect VSWR_limit passed in
        if limit not in (VSWR_LIMIT_NONE, VSWR_LIMIT_6TO1, VSWR_LIMIT_3TO1, VSWR_LIMIT_2TO1):
            return

        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return
        else:
            self._instr.write('VSWR:LIM {}'.format(limit))

    def set_frequency(self, frequency):
        """Sets the frequency for the front panel power indicator correction.
        
        This only applies to amplifiers that have the ability to correct the power indicators and have a valid correction table loaded.

        frequency -- The power correction frequency, in hertz
        """
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return
        else:
            if frequency < 1e6:
                frequency /= 1e3
                unit = 'KHZ'
            elif frequency < 1e9:
                frequency /= 1e6
                unit = 'MHZ'
            else:
                frequency /= 1e9
                unit = 'GHZ'
            self._instr.write('POW:CORR:FREQ {:.3f}{}'.format(frequency, unit))

    def query_rf_gain(self):
        """Queries the current RF gain."""
        if self._commandset == CS_2LINE:
            return 0
        elif self._commandset in (CS_SHORT, CS_4LINE):
            response = self._instr.query('G?')
            if response[0] == 'G' and response.strip() != 'G?':
                return int(int(response[1:]) / 4095 * 100)
        else:
            response = self._instr.query('RFG?')
            if response[0:4] == 'RFG=':
                return int(response.split('=')[1])

        raise QueryError(response)

    def query_alc_detector_gain(self):
        """Queries the current ALC detector gain."""
        if self._commandset in (CS_SHORT, CS_2LINE):
            return 0
        elif self._commandset == CS_4LINE:
            response = self._instr.query('D?')
            if response[0] == 'D' and response.strip() != 'D?':
                return int(int(response[1:]) / 255 * 100)
        else:
            response = self._instr.query('MSB?')
            if response.strip() != 'MSB?' and len(response) >= 10:
                return int(response.split(',')[1].split('=')[1])

        raise QueryError(response)

    def query_alc_response(self):
        """Queries the current ALC response time."""
        if self._commandset in (CS_SHORT, CS_2LINE):
            return 0
        elif self._commandset == CS_4LINE:
            response = self._instr.query('LR?')
            if response[0:2] == 'LR' and response.strip() != 'LR?':
                return int(response[2:])
        else:
            response = self._instr.query('MSB?')
            if response.strip() != 'MSB?' and len(response) >= 10:
                return int(response.split(',')[3].split('=')[1])
        
        raise QueryError(response)

    def query_alc_threshold(self):
        """Queries the current ALC threshold."""
        if self._commandset in (CS_SHORT, CS_2LINE):
            return 0
        elif self._commandset == CS_4LINE:
            response = self._instr.query('T?')
            if response[0] == 'T' and response.strip() != 'T?':
                return int(int(response[1:]) / 4095 * 100)
        else:
            response = self._instr.query('MSB?')
            if response.strip() != 'MSB?' and len(response) >= 10:
                return int(response.split(',')[2].split('=')[1])

        raise QueryError(response)

    def query_forward_power(self):
        """Queries the forward power in watts.

        *Note: On legacy amplifiers, this query returns a value between 0 and either 2048 or 4095.
        Refer to the manual to see the formula needed to convert this number into watts.
        """
        if self._commandset in (CS_SHORT, CS_2LINE):
            return 0
        elif self._commandset == CS_4LINE:
            response = self._instr.query('FP?')
            if response[0:2] == 'FP' and response.strip() != 'FP?':
                return float(response[2:])
        else:
            response = self._instr.query('FPOW?')
            if response[0:5] == 'FPOW=':
                return float(response.split('=')[1])

        raise QueryError(response)

    def query_reverse_power(self):
        """Queries the reverse power in watts.

        *Note: On legacy amplifiers, this query returns a value between 0 and either 2048 or 4095.
        Refer to the manual to see the formula needed to convert this number into watts.
        """
        if self._commandset in (CS_SHORT, CS_2LINE):
            return 0
        elif self._commandset == CS_4LINE:
            response = self._instr.query('RP?')
            if response[0:2] == 'RP' and response.strip() != 'RP?':
                return float(response[2:])
        else:
            response = self._instr.query('RPOW?')
            if response[0:5] == 'RPOW=':
                return float(response.split('=')[1])

        raise QueryError(response)

    def query_operating_hours(self):
        """Queries the operating hours in RF on and power on states of the amplifier.
        
        Refer to the manual to determine if this is supported for your amplifier.
        Returns a tuple in the form (power on hours, RF on hours)
        """
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return 0,0
        else:
            response = self._instr.query('OHP?')
            if response[0:4] == 'OHP=':
                pwr_hours = int(response.split('=')[1])
            else:
                raise QueryError(response)
            response = self._instr.query('OH?')
            if response[0:3] == 'OH=':
                rf_hours = int(response.split('=')[1])
            else:
                raise QueryError(response)

            return pwr_hours, rf_hours

    def query_vswr_limit(self):
        """Queries the value of the output VSWR limit."""
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return 0
        else:
            response = self._instr.query('VSWR:LIM?')
            try:                
                limit = int(response)
            except ValueError:
                raise QueryError(response)
            return limit

    def query_frequency(self):
        """Queries the power indicator correction frequency.
        
        This only applies to amplifiers that have the ability to correct the power indicators and have a valid correction table loaded.
        """
        if self._commandset in (CS_4LINE, CS_SHORT, CS_2LINE):
            return 0
        else:
            response = self._instr.query('CORR:POW:FREQ?')
            m = re.search(r'([0-9\.]+)([KMG]HZ)', response)
            if m:
                if m.group(2) == 'GHZ':
                    frequency = m.group(1) * 1e9
                elif m.group(2) == 'MHZ':
                    frequency = m.group(1) * 1e6
                else:
                    frequency = m.group(1) * 1e3
                return frequency
            else:
                raise QueryError(response)
        
        return 0
