"""
AR Field Probe module

Copyright 2019 Amplifier Research
"""

import time
import visa
import re
import enum
from .errors import QueryError, ParameterError


class AR_FieldProbe:
    """Class for handling VISA communication with FL series field probes."""
    def initialize(self, VISAResource,
                   baudRate=19200,
                   flowControl=visa.constants.VI_ASRL_FLOW_NONE,
                   parity=visa.constants.VI_ASRL_PAR_NONE,
                   dataBits=8,
                   stopBits=visa.constants.VI_ASRL_STOP_ONE):
        """Opens a VISA session to the probe.

        If VISAResource is a serial port, it will be configured using the supplied parameters.
        flowControl, parity, and stopBits should use constants from visa.constants.
        """
        self._instr = visa.ResourceManager().open_resource(VISAResource, query_delay=0.05, write_termination='\r')

        self._instr.read_termination = '\r'

        # Configure serial if necessary
        if self._instr.interface_type == visa.constants.InterfaceType.asrl:
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_BAUD, baudRate)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_FLOW_CNTRL, flowControl)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_PARITY, parity)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_DATA_BITS, dataBits)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_STOP_BITS, stopBits)

    def close(self):
        """Closes the VISA session."""
        self._instr.close()

    def query_probe_id(self):
        """Queries information about the attached probe.

        Returns a dictionary of strings with model, serial, firmware, linearization, and status_ok keys.
        """
        response: str = self._instr.query('I')
        if response[0:2] == ':I':
            splitresponse = response.split(',')
            return dict(model=splitresponse[1].strip(),
                        serial=splitresponse[2].strip(),
                        firmware=splitresponse[3].strip(),
                        linearization=splitresponse[4].strip(),
                        status_ok=splitresponse[5].strip() == 'S')
        else:
            raise QueryError(response.strip())

    def query_interface_id(self):
        """Queries information about the probe interface box.

        Returns a dictionary of strings with model, serial, and firmware keys.
        """
        response: str = self._instr.query('*IDN?')
        if response.count(',') == 2:
            splitresponse = response.split(',')
            return dict(model=splitresponse[1].strip(),
                        serial=self._instr.query('SN?').strip(),
                        firmware=splitresponse[2].strip())
        else:
            raise QueryError(response.strip())

    def read_data(self):
        """Takes a reading from the probe.

        Returns data in a dictionary with keys x, y, z, c that contain the reading on each axis and the composite value.
        There is also a status_ok key which is True or False as explained in the probe manual.
        """
        response: str = self._instr.query('D')
        if response[0:2] == ':D':
            return dict(x=float(response[2:7]),
                        y=float(response[7:12]),
                        z=float(response[12:17]),
                        c=float(response[17:22]),
                        status_ok=(response[22] == 'S'))
        else:
            raise QueryError(response.strip())


class AR_FieldMonitor:
    """Class for handling VISA communication with the FM7004(A) field monitor."""
    class DisplayMode(enum.IntEnum):
        """Enumeration of available display modes."""
        XYZ = 0
        MMA = 1
        MMH = 2

    def initialize(self, VISAResource,
                   baudRate=19200,
                   flowControl=visa.constants.VI_ASRL_FLOW_NONE,
                   parity=visa.constants.VI_ASRL_PAR_NONE,
                   dataBits=8,
                   stopBits=visa.constants.VI_ASRL_STOP_ONE):
        """Opens a VISA session to the probe.

        If VISAResource is a serial port, it will be configured using the supplied parameters.
        flowControl, parity, and stopBits should use constants from visa.constants.
        """
        self._instr = visa.ResourceManager().open_resource(VISAResource, query_delay=0.05, write_termination='\r')

        self._instr.read_termination = '\r'

        # Configure serial if necessary
        if self._instr.interface_type == visa.constants.InterfaceType.asrl:
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_BAUD, baudRate)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_FLOW_CNTRL, flowControl)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_PARITY, parity)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_DATA_BITS, dataBits)
            self._instr.set_visa_attribute(visa.constants.VI_ATTR_ASRL_STOP_BITS, stopBits)

    def reset(self):
        """Resets the FM and clears errors."""
        self._instr.write('*RST')
        time.sleep(5)

    def local_mode(self):
        """Returns the FM to local mode."""
        self._instr.write('LOCL')

    def close(self):
        """Closes the VISA session."""
        self._instr.close()

    def config_display_mode(self, mode: DisplayMode):
        """Configures the front panel display mode.

        The mode parameter is an AR_FieldMonitor.DisplayMode enumeration."""
        if mode == self.DisplayMode.MMH:
            self._instr.write('DISP,MMH')
        elif mode == self.DisplayMode.MMA:
            self._instr.write('DISP,MMA')
        else:
            self._instr.write('DISP,XYZ')
        time.sleep(0.5)

    def config_frequency_correction(self, state):
        """Configures the state of internal frequency correction.

        The state parameter will accept 1, 'on', or True to enable correction.
        """
        if state in (1, True, 'ON', 'On', 'on'):
            command = 'CORR,ON'
        else:
            command = 'CORR,OFF'
        self._instr.write(command)
        time.sleep(0.1)

    def set_frequency(self, frequency):
        """Sets the frequency used for internal correction.

        The frequency parameter should be given in Hertz.
        """
        if frequency > 999.999e9:
            frequency = 999.999e9
        if frequency >= 1e9:
            frequency /= 1e9
            unit = 'G'
        elif frequency >= 1e6:
            frequency /= 1e6
            unit = 'M'
        else:
            frequency /= 1e3
            unit = 'K'

        self._instr.write(f'FREQ,{frequency:07.3f}{unit}')
        time.sleep(0.1)

    def run_mmh(self):
        """Runs the min/max hold feature when the display is set to min/max hold."""
        self._instr.write('MMH,RUN')

    def stop_mmh(self):
        """Stops the min/max hold feature when the display is set to min/max hold."""
        self._instr.write('MMH,STOP')

    def config_channel_state(self, channel, state):
        """Enables or disables the specified channel.

        The state parameter will accept 1, 'on', or True to enable the channel.
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')

        if state in (1, True, 'ON', 'On', 'on'):
            self._instr.write('CH,{channel},ST,ON')
            time.sleep(1)
        else:
            self._instr.write(f'CH,{channel},ST,OFF')
            time.sleep(0.05)

    def config_sample_rate(self, channel, rate):
        """Configures the sample rate of the specified channel.

        The sample rate parameter should adhere to the following:
        FL/PL/FA 7000 series:
            x = Sample Rate 5 to 25 (Increments of 5)
        FP/FH 7000 series:
            x = Sample Rate 5 to 40 (Increments of 5)
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')

        # Clamp the rate to the valid range and round it to the nearest increment of 5
        rate = max(5, min(40, rate))
        rate = round(rate / 5) * 5

        self._instr.write(f'CH,{channel},SR,{rate}')
        time.sleep(0.1)

    def config_axes(self, channel, x, y, z):
        """Configure axes used for composite calculation for the specified channel.

        The x, y, and z parameters will accept 1, 'on', or True to enable each axis.
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')

        x = '1' if x in (1, True, 'ON', 'On', 'on') else '0'
        y = '1' if y in (1, True, 'ON', 'On', 'on') else '0'
        z = '1' if z in (1, True, 'ON', 'On', 'on') else '0'

        self._instr.write(f'CH,{channel},AX,{x}{y}{z}')
        time.sleep(0.1)

    def zero_probe(self, channel):
        """Zeros the probe on the specified channel."""
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')

        self._instr.write(f'CH,{channel},ZP')
        time.sleep(1)

    def id_query(self):
        """Returns the identity string from the FM."""
        return self._instr.query('*IDN?').strip()

    def query_display_mode(self):
        """Queries the current display mode.

        Returns an AR_FieldMonitor.DisplayMode enumeration."""
        response: str = self._instr.query('DISP?')
        if response.find('MMH') >= 0:
            return self.DisplayMode.MMH
        elif response.find('MMA') >= 0:
            return self.DisplayMode.MMA
        elif response.find('XYZ') >= 0:
            return self.DisplayMode.XYZ
        else:
            raise QueryError(response)

    def query_frequency_correction(self):
        """Queries the state of internal frequency correction.

        Returns True or False to indicate the state.
        """
        response: str = self._instr.query('CORR?')
        if response.find('CORR') >= 0:
            return response.find('ON') >= 0
        else:
            raise QueryError(response)

    def query_frequency(self):
        """Queries the current frequency used for internal frequency correction.

        Returns the frequency as a floating point value in Hertz.
        """
        response: str = self._instr.query('FREQ?')
        if response[0:4] == 'FREQ':
            frequency = float(response[5:12])
            if response[12] == 'G':
                frequency *= 1e9
            elif response[12] == 'M':
                frequency *= 1e6
            else:
                frequency *= 1e3
            return frequency
        else:
            raise QueryError(response)

    def query_MMH_state(self):
        """Queries the running state of the min/max hold feature.

        Returns True if min/max hold is running, False if it is not.
        """
        response: str = self._instr.query('MMH?')
        return response.upper().find('RUNNING') >= 0

    def query_error_log(self):
        """Queries the error log.

        Returns a dictionary with the following keys:
        error_no: error number, 0 for none
        error: error string
        channel: the affected channel
        """
        response: str = self._instr.query('ERR?')
        if response.find('NONE') >= 0:
            return dict(error_no=0,
                        error='NONE',
                        channel=0)
        elif response[0:5] == 'ERR,E':
            n = response[5]
            if n == '1':
                errortxt = 'COMM LOST'
            else:
                errortxt = 'I/O ERROR'
            return dict(error_no=int(n),
                        error=errortxt,
                        channel=response[9])
        else:
            raise QueryError(response)

    def query_channel_state(self, channel):
        """Queries the state of the specified channel.

        Returns True or False to indicate if the channel is on or off.
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')
        response: str = self._instr.query(f'CH,{channel},ST?')
        if response[0:7] == f'CH,{channel},ST':
            return response.find('ON') >= 0
        else:
            raise QueryError(response)

    def query_sample_rate(self, channel):
        """Queries the sample rate of the specified channel."""
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')
        response: str = self._instr.query(f'CH,{channel},SR?')
        if response[0:7] == f'CH,{channel},SR':
            return int(response[8:].strip())
        else:
            raise QueryError(response)

    def query_axes(self, channel):
        """Queries the axes used for composite calculation for the specified channel.

        Returns a dictionary with x, y, and z keys that are set to True or False.
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')
        response: str = self._instr.query(f'CH,{channel},AX?')
        if response.startswith(f'CH,{channel},AX'):
            return dict(x=response[8] == '1',
                        y=response[9] == '1',
                        z=response[10] == '1')
        else:
            raise QueryError(response)

    def query_probe_status(self, channel):
        """Queries the status of the probe on the specified channel.

        Returns a dictionary with the following keys:
        model: the probe model number
        serial: the probe serial number
        firmware: the probe firmware revision
        status_ok: the probe battery/laser status (True if battery/laser is ok, False for low voltage/)
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')
        response: str = self._instr.query(f'CH,{channel},PS?')
        if response.startswith(f'CH,{channel},PS'):
            responselist = response.split(',')
            return dict(model=responselist[3].strip(),
                        serial=responselist[4].strip(),
                        firmware=responselist[5].strip(),
                        status_ok=responselist[6].strip() == 'OK')
        else:
            raise QueryError(response)

    def read_data(self, channel):
        """Takes a reading from the probe on the specified channel.

        Returns data in a dictionary with keys x, y, z, c that contain the reading on each axis and the composite value.
        Other keys in the dictionary that indicate status are as follows:
        in_range: True if the reading is within the specified linear operating range of the probe
        freq_in_range: True if the selected frequency is within the range of the stored correction factor table
        status_ok: True if the battery/laser is in good condition
        """
        if channel not in range(1, 5):
            raise ParameterError(f'{channel} is not a valid channel.')
        response: str = self._instr.query(f'DS,{channel}?')
        if response.startswith(f'DS,{channel},'):
            responselist = response.split(',')
            return dict(x=float(responselist[2]),
                        y=float(responselist[3]),
                        z=float(responselist[4]),
                        c=float(responselist[5]),
                        in_range=responselist[6] == '1',
                        freq_in_range=responselist[7] == '1',
                        status_ok=responselist[8] == '1')
        else:
            raise QueryError(response)

    def read_display(self):
        """Takes a reading from the display of the FM. The data that is returned depends on the active display mode.

        Returns a dictionary containing the valid measurements in the following keys for the active display mode.
        Display mode: XYZ
            c1: channel 1 composite value
            c2: channel 2 composite value
            c3: channel 3 composite value
            c4: channel 4 composite value
        Display mode: MMA
            min: minimum composite value of all enabled channels
            max: maximum composite value of all enabled channels
            avg: average composite value of all enabled channels
        Display mode: MMH
            min_hold: minimum composite value of all enabled channels since calling run_MMH()
            max_hold: maximum composite value of all enabled channels since calling run_MMH()
        """
        response: str = self._instr.query('RD?')
        responselist = response.split(',')
        if len(responselist) == 5:
            return dict(c1=float(responselist[1]) if responselist[1].find('---') == -1 else 0,
                        c2=float(responselist[2]) if responselist[2].find('---') == -1 else 0,
                        c3=float(responselist[3]) if responselist[3].find('---') == -1 else 0,
                        c4=float(responselist[4]) if responselist[4].find('---') == -1 else 0)
        elif len(responselist) == 4:
            return dict(min=float(responselist[1]),
                        max=float(responselist[2]),
                        avg=float(responselist[3]))
        elif len(responselist) == 3:
            return dict(min_hold=float(responselist[1]),
                        max_hold=float(responselist[2]))
        else:
            raise QueryError(response)


class AR_FieldAnalyzer:
    """Class for handling VISA communication with FA series field analyzers."""
    class Units(enum.IntEnum):
        """Enumeration of availabe units."""
        Vm = 0
        mWcm2 = 1
        Wm2 = 2

    class TriggerMode(enum.IntEnum):
        """Enumeration of available trigger modes."""
        FREERUN = 0
        INTERNAL = 1
        EXTERNAL = 2

    class TriggerEdge(enum.IntEnum):
        """Enumeration of available trigger edge values."""
        RISING = 0
        FALLING = 1

    class AddressMode(enum.IntEnum):
        """Enumeration of available IP address modes."""
        DYNAMIC = 0
        STATIC = 1

    def initialize(self, VISAResource):
        """Opens a VISA session to the analyzer."""
        self._instr = visa.ResourceManager().open_resource(VISAResource, query_delay=0.05, write_termination='\n')

        self._instr.read_termination = '\n'

    def reset(self):
        """Resets the FA and clears errors."""
        self._instr.write('*RST')
        time.sleep(5)

    def close(self):
        """Closes the VISA session."""
        self._instr.close()

    def id_query(self):
        """Returns the identity string from the FA."""
        return self._instr.query('*IDN?').strip()

    def config_amplitude_scale(self, scale, unit: Units=Units.Vm):
        """Configures the amplitude scale with the given units per division.

        Note: This function does not change the units. That must be done separately with config_units()

        scale: the scale value per division in the given units
        units: an AR_FieldAnalyzer.Units enumeration, default of V/m
        """
        # Convert to the V/m column of the table
        if unit == 1:
            vmscale = scale * 100
        elif unit == 2:
            vmscale = scale * 10
        else:
            vmscale = scale

        # For each group of 3 in the table
        for i in range(5):
            vmscalelist = [0.1 * 10 ** i, 0.2 * 10 ** i, 0.5 * 10 ** i]
            # For each scale within a group of 3
            for j in range(3):
                # Check if the given value is less than any in the group
                if vmscale <= vmscalelist[j]:
                    # The index is the group number + the element within the group of 3
                    index = i * 3 + j
                    self._instr.write(f'SL{index:02}')
                    return

        # Given scale is greater than all values in the table
        self._instr.write(f'SL14')
        time.sleep(0.1)

    def config_time_base(self, time_base):
        """Configures the time base in microseconds.

        time_base: time base in microseconds/division.
        """
        # For each group of 3 in the table
        for i in range(3):
            us = [1 * 10 ** i, 2 * 10 ** i, 4 * 10 ** i]
            # For each time base within a group of 3
            for j in range(3):
                # Check if the given value is less than any in the group
                if time_base <= us[j]:
                    # The index is the group number + the element within the group of 3
                    index = i * 3 + j
                    self._instr.write(f'TB{index:02}')
                    return

        # Given time base is greater than all values in the table
        self._instr.write(f'TB08')
        time.sleep(0.1)

    def config_trigger_mode(self, mode: TriggerMode):
        """Configures the trigger mode.

        mode: an AR_FieldAnalyzer.TriggerMode enumeration.
        """
        self._instr.write(f'TM{mode}')
        time.sleep(0.1)

    def config_trigger_threshold(self, level):
        """Configures the internal trigger threshold."""
        self._instr.write(f'TL{max(0,level)}')
        time.sleep(0.1)

    def config_trigger_edge(self, edge: TriggerEdge):
        """Configures the internal trigger edge (rising or falling).

        edge: an AR_FieldAnalyzer.TriggerEdge enumeration."""
        self._instr.write(f'TE{edge}')
        time.sleep(0.1)

    def config_offset_from_trigger(self, microseconds):
        """Configures the offset in microseconds to lead or lag the trigger point.

        This offset value is used to set the moment in time to take a measurement using the
        Single Sample Composite Field query. It can be any value between -2000.00 and 2000.00.
        If the Time Base is too low for the offset entered, an error will be caused when the composite field is read.
        """
        microseconds = min(2000, max(-2000, microseconds))
        self._instr.write(f'SO{microseconds}')
        time.sleep(0.1)

    def config_units(self, units: Units):
        """Configures the units on the FA.

        units: an AR_FieldAnalyzer.Units enumeration.
        """
        self._instr.write(f'UNITS{units}')
        time.sleep(0.1)

    def config_ip_address_mode(self, mode: AddressMode):
        """Configures the IP address mode to static or dynamic.

        mode: an AR_FieldAnalyzer.AddressMode enumeration.
        """
        if mode == self.AddressMode.STATIC:
            self._instr.write('IPM,S')
        else:
            self._instr.write('IPM,D')
        time.sleep(0.1)

    def config_static_ip_address(self, address, subnet_mask='255.255.000.000'):
        """Configures the static IP address and subnet mask.

        The subnet mask defaults to 255.255.000.000.
        """
        # Format addresses to make each octet 3 characters wide
        address = '.'.join('{0:03}'.format(int(i)) for i in address.split('.'))
        subnet_mask = '.'.join('{0:03}'.format(int(i)) for i in subnet_mask.split('.'))

        self._instr.write('SIP,' + address)
        time.sleep(0.05)
        self._instr.write('SSM,' + subnet_mask)
        time.sleep(0.05)

    def query_probe_id(self):
        """Queries information about the attached probe.

        Returns a dictionary of strings with model, serial, firmware, linearization, and status_ok keys.
        """
        response: str = self._instr.query('I')
        if response[0:2] == ':I':
            splitresponse = response.split(',')
            return dict(model=splitresponse[1].strip(),
                        serial=splitresponse[2].strip(),
                        firmware=splitresponse[3].strip(),
                        linearization=splitresponse[4].strip(),
                        status_ok=splitresponse[5].strip() == 'S')
        else:
            raise QueryError(response.strip())

    def query_status(self):
        """Queries the system status.

        Returns a dictionary with the following True/False keys:
        fault: system fault present
        laser: laser currently active
        key_position: key in the 'enabled' position
        triggered: trigger condition has been met
        web_active: web application is currently running
        sync_error: error syncing data with the field sensor
        int_com_error: internal communications error
        """
        try:
            response: str = self._instr.query('STATUS?')
            status = int.from_bytes(response[0].encode(), byteorder='big')
        except ValueError:
            raise QueryError(response)

        return dict(fault=status & 1 != 0,
                    laser=status & 2 != 0,
                    key_position=status & 4 != 0,
                    triggered=status & 8 != 0,
                    web_active=status & 16 != 0,
                    sync_error=status & 32 != 0,
                    int_com_error=status & 128 != 0)

    def query_amplitude_scale(self):
        """Queries the amplitude scale.

        Returns an index. Refer to the manual for the corresponding amplitude scale.
        """
        response: str = self._instr.query('SL?')
        if response.startswith('SL'):
            return int(response[2:4])
        else:
            raise QueryError(response)

    def query_time_base(self):
        """Queries the time base.

        Returns an index. Refer to the manual for the corresponding time base.
        """
        response: str = self._instr.query('TB?')
        if response.startswith('TB'):
            return int(response[2:4])
        else:
            raise QueryError(response)

    def query_trigger_mode(self):
        """Queries the trigger mode.

        Returns an AR_FieldAnalyzer.TriggerMode enumeration.
        """
        response: str = self._instr.query('TM?')
        if response.startswith('TM'):
            return self.TriggerMode(int(response[2:4]))
        else:
            raise QueryError(response)

    def query_trigger_threshold(self):
        """Queries the internal trigger threshold.

        Returns a float representing the threshold."""
        response: str = self._instr.query('TL?')
        if response.startswith('TL'):
            return float(response[2:].strip())
        else:
            raise QueryError(response)

    def query_trigger_edge(self):
        """Queries the internal trigger edge.

        Returns an AR_FieldAnalyzer.TriggerEdge enumeration.
        """
        response: str = self._instr.query('TE?')
        if response.startswith('TE'):
            return self.TriggerEdge(int(response[2]))
        else:
            raise QueryError(response)

    def query_offset_from_trigger(self):
        """Queries the time in microseconds to lead or lag the trigger point.

        Returns a float representing the offset.
        """
        response: str = self._instr.query('SO?')
        if response.startswith('SO'):
            return float(response[2:].strip())
        else:
            raise QueryError(response)

    def query_units(self):
        """Queries the current units.

        Returns an AR_FieldAnalyzer.Units enumeration.
        """
        response: str = self._instr.query('UNITS?')
        if response.startswith('UNITS'):
            return self.Units(int(response[5]))
        else:
            raise QueryError(response)

    def query_ip_address_mode(self):
        """Queries the IP address mode.

        Returns a dictionary with the following keys:
        mode: an AR_FieldAnalyzer.AddressMode enumeration
        address: the current IP address.
        """
        response: str = self._instr.query('IPM?')
        if response.startswith('IPM'):
            splitresponse = response.split(',')
            return dict(mode=self.AddressMode.STATIC if splitresponse[1] == 'S' else self.AddressMode.DYNAMIC,
                        address=splitresponse[2].strip())
        else:
            raise QueryError(response)

    def query_static_ip_address(self):
        """Queries the static IP address and subnet mask.

        Returns a dictionary with keys address and subnet_mask.
        """
        response: str = self._instr.query('SIP?')
        if response.startswith('SIP'):
            sip = response[4:].strip()
            response: str = self._instr.query('SSM?')
            if response.startswith('SSM'):
                ssm = response[4:].strip()
            else:
                raise QueryError(response)
        else:
            raise QueryError(response)

        return dict(address=sip, subnet_mask=ssm)

    def read_composite_field(self):
        """Reads a single sample of the composite field or power density.

        Returns a single float representing the composite field level or power density,
        depending on the units selected, at the offset trigger point set by config_offset_from_trigger().
        """
        response: str = self._instr.query('SMP?')
        if len(response) >= 5 and response.find('.') >= 0:
            return float(response)
        else:
            raise QueryError(response)

    def read_mma(self):
        """Reads the minimum, maximum, and average composite field level or power density,
        depending on the units selected, over the period of time that is ten times the selected time base.

        Returns a dictionary of floating point values with keys min, max, and avg."""
        response: str = self._instr.query('MMA?')
        mma = [float(i) for i in response.strip().split(',')]
        if len(mma) == 3:
            return dict(min=mma[0],
                        max=mma[1],
                        avg=mma[2])
        else:
            raise QueryError(response)